local debugWorldDraw_scissor_bodies
local function debugWorldDraw_scissor_callback(fixture)
	debugWorldDraw_scissor_bodies[#debugWorldDraw_scissor_bodies+1] = fixture:getBody()
	return true --search continues until false
end

local function debugWorldDraw(world,x,y)
	love.graphics.push()
	
	local bodies
	if (x) then
		local width,height = love.window.getMode()
		love.graphics.translate(-x+width/2,-y+height/2) --center
	
		debugWorldDraw_scissor_bodies = {}
		world:queryBoundingBox(x-width/2,y-height/2,x+width/2,y+height/2,debugWorldDraw_scissor_callback) --draw only bodies intersecting screen
		bodies = debugWorldDraw_scissor_bodies
	else
		bodies = world:getBodyList()
	end
	
	for b=#bodies,1,-1 do
		local body = bodies[b]
		local bx,by = body:getPosition()
		local bodyAngle = body:getAngle()
		love.graphics.push()
		love.graphics.translate(bx,by)
		love.graphics.rotate(bodyAngle)
		
		math.randomseed(1) --for color generation
		
		local fixtures = body:getFixtureList()
		for i=1,#fixtures do
			local fixture = fixtures[i]
			local shape = fixture:getShape()
			local shapeType = shape:getType()
			local isSensor = fixture:isSensor()
			
			if (isSensor) then
				love.graphics.setColor(0,0,255,96)
			else
				love.graphics.setColor(math.random(32,200),math.random(32,200),math.random(32,200),96)
			end
			
			love.graphics.setLineWidth(1)
			if (shapeType == "circle") then
				local x,y = fixture:getMassData() --0.9.0 missing circleshape:getPoint()
				--local x,y = shape:getPoint() --0.9.1
				local radius = shape:getRadius()
				love.graphics.circle("fill",x,y,radius,15)
				love.graphics.setColor(0,0,0,255)
				love.graphics.circle("line",x,y,radius,15)
				local eyeRadius = radius/4
				love.graphics.setColor(0,0,0,255)
				love.graphics.circle("fill",x+radius-eyeRadius,y,eyeRadius,10)
			elseif (shapeType == "polygon") then
				local points = {shape:getPoints()}
				love.graphics.polygon("fill",points)
				love.graphics.setColor(0,0,0,255)
				love.graphics.polygon("line",points)
			elseif (shapeType == "edge") then
				love.graphics.setColor(0,0,0,255)
				love.graphics.line(shape:getPoints())
			elseif (shapeType == "chain") then
				love.graphics.setColor(0,0,0,255)
				love.graphics.line(shape:getPoints())
			end
		end
		love.graphics.pop()
	end
	
	local joints = world:getJointList()
	for index,joint in pairs(joints) do
		love.graphics.setColor(0,255,0,255)
		local x1,y1,x2,y2 = joint:getAnchors()
		if (x1 and x2) then
			love.graphics.setLineWidth(3)
			love.graphics.line(x1,y1,x2,y2)
		else
			love.graphics.setPointSize(3)
			if (x1) then
				love.graphics.point(x1,y1)
			end
			if (x2) then
				love.graphics.point(x2,y2)
			end
		end
	end
	
	local contacts = world:getContactList()
	for i=1,#contacts do
		love.graphics.setColor(255,0,0,255)
		love.graphics.setPointSize(3)
		local x1,y1,x2,y2 = contacts[i]:getPositions()
		if (x1) then
			love.graphics.point(x1,y1)
		end
		if (x2) then
			love.graphics.point(x2,y2)
		end
	end
	
	debugWorldDraw_scissor_bodies = nil
	love.graphics.pop()
end


require("lovelymoon/stateManager")
require("lovelymoon/lovelyMoon")
require("maze")

function love.load()
  -- Basic config
  debug = true
  
  -- Set the physics properties
  love.physics.setMeter(32)
  world = love.physics.newWorld(0*love.physics.getMeter(), 0*love.physics.getMeter())
  
  
  addState(MazeState, "maze")
  
  enableState("maze")
end

function love.update(dt)
  lovelyMoon.update(dt)
end

function love.keypressed(key, unicode)
  lovelyMoon.keypressed(key, unicode)
end

function love.resize()
  windowWidth, windowHeight = love.window.getMode()
  
  if isStateEnabled("game") then
	 map:resize()
  end
end

function love.draw()
  --lovelyMoon.draw()
	debugWorldDraw(world,Player.body:getWorldCenter())
end

function love.mousepressed(x, y, button)
  lovelyMoon.mousepressed(x, y, button)
end

function love.mousereleased(x, y, button)
  lovelyMoon.mousereleased(x, y, button)
end

function love.keypressed(key, unicode)
  lovelyMoon.keypressed(key, unicode)
end

function love.keyreleased(key, unicode)
  lovelyMoon.keyreleased(key, unicode)
end