require('player')

MazeState = {}

math.randomseed(os.time())

local p

-- Map variables
local map = {}
local walls = {}
local cells = {}
local x, y = 0, 0
local wall = {}

--New
function MazeState:new()
   local gs = {}

   gs = setmetatable(gs, self)
   self.__index = self
   _gs = gs
   
   return gs
end

--Load
function MazeState:load()
  setupMap()
  
  -- Get starting cell
  x = 1 + 2*math.floor(math.random()*((map_w-1)/2+1))
  y = 1 + 2*math.floor(math.random()*((map_h-1)/2+1))
  -- Mark the starting cell as part of the maze
  map[y][x].t = 0
  -- Add starting cell to cell table
  table.insert(cells, {x= x, y= y})
  
  -- Add starting cell walls to wall table
  get_walls()
  
  while table.getn(walls)>0 do
    -- Select random wall from wall table
    wall = walls[math.random(#walls)]
    
    -- Check if the opposite cell to the current one, over the wall, is part of the maze
    check_wall()
    
    -- Remove the wall from the table
    for i, w in ipairs(walls) do
      if w.x==wall.x and w.y==wall.y then
        table.remove(walls, i)
        break
      end
    end
    
  end -- End while
  
  -- Add phyisics to the walls
  for c, col in ipairs(map) do
    for r, cell in ipairs(col) do
      if map[c][r].t == 1 then
        map[c][r].body = love.physics.newBody(world, (r-1)*tile_w+tile_w/2, (c-1)*tile_h+tile_h/2, 'static')
        map[c][r].shape = love.physics.newRectangleShape(tile_w, tile_h)
        love.physics.newFixture(map[c][r].body, map[c][r].shape) 
      end
    end
  end
  
  -- Add player position
  for r, _ in ipairs(map[1]) do
    if map[2][r].t==0 then
      -- Non wall found add player
      local px = (r-1)*tile_w+(tile_w/4)
      local py = (tile_h/4)
      -- Load player into map
      getPlayer(px, py)
      break
    end
  end
end -- End load

function setupMap()
  -- Map conf
  map_h = 151 -- Amount of tiles
  map_w = 151 -- Amount of tiles
  map_x = 0
  map_y = 0
  map_display_buffer = 2 -- Stop tiles 'popping' in
  
  -- Display settings
  winW, winH = love.window.getMode()
  tile_w = 64 -- Pixels
  tile_h = 64 -- Pixels
  map_display_w = winW/tile_w
  map_display_h = winH/tile_h
  
  -- Map tiles
  tile = {}
  tile[0] = love.graphics.newImage('images/floor.png')
  tile[1] = love.graphics.newImage('images/wall.png')
  
  -- Generate map full of walls
  for col=1, map_h do
    map[col] = {}
    for row=1, map_w do
      map[col][row] = {}
      map[col][row].t = 1
    end
  end
end

-- ************************** Wall Functions **************************
function get_walls()
  -- Gets the surrounding walls of the current cell
  -- Add these to the wall table with the next cell to move to
  
  --print(x..","..y)
  
  -- Can we go up
  if y>1 then
    table.insert(walls, {x= x, y= y-1, cellx= x, celly= y-2})
  end
  -- Can we go down
  if y<map_h-1 then
    table.insert(walls, {x= x, y= y+1, cellx= x, celly= y+2})
  end
  -- Can we go left
  if x>1 then
    table.insert(walls, {x= x-1, y= y, cellx= x-2, celly= y})
  end
  -- Can we go right
  if x<map_w-1 then
    table.insert(walls, {x= x+1, y= y, cellx= x+2, celly= y})
  end
end

function check_wall()
  -- Checks if opposite cell to current is in the maze
  
  -- Get the next cell
  -- Check if in cells
  local is_in = false
  for _, c in ipairs(cells) do
    if c.x==wall.cellx and c.y==wall.celly then
      -- Cell already visited
      is_in = true
      break
    end
  end
  
  -- If cell not visited yet
  if not is_in then
    -- Set the next cell as current 
    x, y = wall.cellx, wall.celly
    -- Add it to the cell table
    table.insert(cells, {x= x, y= y})
    -- Break the wall between the cells
    map[wall.y][wall.x].t = 0
    map[y][x].t = 0
    -- Add the next cell walls to the wall table
    get_walls()
  end
end
-- ************************** Wall Functions **************************

--Close
function MazeState:close()
end

--Enable
function MazeState:enable()
end

--Disable
function MazeState:disable()
end

--Update
function MazeState:update(dt)
  -- Update the player and the world
  Player.update(dt)
  world:update(dt)
  
  -- Update the map position relative to the player position
  -- Map should stop moving when it reaches edges
  winW, winH = love.window.getMode()
  -- Player centered
  local mx, my = Player.x-(winW/2), Player.y-(winH/2)
  if mx<0 then
    -- Map at far left
    map_x = 0
  elseif mx>map_w*tile_w+map_display_w then
    -- Map at far right
    map_x = map_w*tile_w+map_display_w
  else
    map_x = mx
  end
  
  if my<0 then
    -- Map at top
    map_y = 0
  elseif my>map_h*tile_h+map_display_h then
    -- Map at bottom
    map_y = map_h*tile_h+map_display_h
  else
    map_y = my
  end
end

--Draw
function MazeState:draw()
  love.graphics.setBackgroundColor(240, 230, 140)
  offset_x = map_x % tile_w
  offset_y = map_y % tile_h
  firstTile_x = math.floor(map_x / tile_w)
  firstTile_y = math.floor(map_y / tile_h)
  
  love.graphics.translate(-map_x, -map_y)
  
  --Draw the map
  for y=1, (map_display_h + map_display_buffer) do
    for x=1, (map_display_w + map_display_buffer) do
      -- Note that this condition block allows us to go beyond the edge of the map.
      if y+firstTile_y >= 1 and y+firstTile_y <= map_h
        and x+firstTile_x >= 1 and x+firstTile_x <= map_w
      then
        local m = map[y+firstTile_y][x+firstTile_x]
        
        love.graphics.draw(
          tile[m.t], 
          ((x-1)*tile_w) - offset_x, 
          ((y-1)*tile_h) - offset_y)
        -- If there is a wall, move it to the same position as the tile
        if m.body then
          m.body:setPosition(((x-1)*tile_w) - offset_x+32, ((y-1)*tile_h) - offset_y+32)
        end
        
        -- Draw physics bodies
        if m.body and debug then
          love.graphics.polygon("line", m.body:getWorldPoints(m.shape:getPoints()))
        end
      end -- end if
    end -- end for
  end -- end for
  
  -- Draw the player
  Player.draw()
  if debug then
    love.graphics.print(math.floor(map_x)..", "..math.floor(map_y), map_x+10, map_y+10)
    love.graphics.print(firstTile_y..", "..firstTile_x, map_x+10, map_y+30)
  end
end

--KeyPressed
function MazeState:keypressed(key, unicode)
end

--KeyReleased
function MazeState:keyreleased(key, unicode)
end

--MousePressed
function MazeState:mousepressed(x, y, button)
end

--MouseReleased
function MazeState:mousereleased(x, y, button)
end